if (!Hijax) throw "Requires: [Hijax]";
/**
 * Hijax.Abstract 
 * 
 * Main Hijax implementation which is expected to be extended by Hijax.HTML5 and 
 * Hijax.Old below
 * 
 * @package 	Hijax
 * @depends		Hijax
 */
Hijax.Abstract = new Class({
	
	Implements: [Options],

	options: {
		home: '',
		idPrefix: '',
		ajaxBaseUrl: '/pages/',
		hasDynamicContent: []
	},
	
	referrer: '/',
	
	// Flag to tell us once the first state change has occurred
	initial: true,
	
	// Stores a pointer to the current request so it can be cancelled if needed
	request: null,
	
	pendingStateChange: null,
	
	/**
	 * Constructor.
	 * Initialises the underlying history mechanism.
	 */
	initialize: function(options)
	{
		this.setOptions(options);
		
		/**
		 * Set up a Mootools Pseudo Selector used to scan for <a href=...>
		 */
		Selectors.Pseudo.hijackable = function()
		{
			var el = $(this);
			return !! el.get('href') && el.get('data-hijack') != 'no';
		};
		
		// Deal with a pending state change if there is one
		this.handlePageLoaded();
	},
	
	handlePageLoaded: function()
	{
		var handler = function(url)
		{
			// If there is a pending state change
			if (this.pendingStateChange)
			{
				Hijax.log.info('Pending state change: '+this.pendingStateChange);
				
				// If the pending state change doesn't matches the loaded url
				if (this.pendingStateChange !== '/'+url)
				{
					this.onStateChanged(this.pendingStateChange);
				}
				else
				{
					Hijax.log.info(
						'Pending state change ignored because it matches: '+url);
				}
			}
			this.pendingStateChange = null;
			
		}.bind(this);
		
		window.addEvent('pageloaded', handler);
	},
	
	setPendingStateChange: function(mappedURL)
	{
		this.pendingStateChange = mappedURL;
	},
	
	start: function()
	{
		/**
		 * Before getting the system started, first make sure that you're on "/" 
		 * (applies to Old mode only)
		 */
		if (!this.redirect())
		{
			this.initHistory();
			this.hijack();
		}
	},
	
	/**
	 * Scan and hijack the anchors in an element.
	 * If no parent element is specified then the entire document is scanned.
	 */
	hijack: function(parent)
	{
		var elem = parent ? $(parent) : document;
		var anchors = elem.getElements('a:hijackable');
		this.hijackAnchors(anchors);
		window.fireEvent('hijacked', anchors);
	},
	
	hijackAnchors: function(anchors)
	{
		// Obtain the http://example.com part, which is needed for IE6/7 and any
		// other browser which reports absolute URLs instead of relative in "href".
		var siteURL = window.location.protocol+'//'+window.location.host;
		
		anchors.addEvent('click', function(e){
			if (!this.isExternal(e))
			{
				e.preventDefault();
				
				var href = this.getURL(e);
				
				// Strip the http://example.com part so you are left with /rates/diving
				href = href.replace(siteURL, '');
				
				// Remove # if there is one
				var url = href.replace(/^#/, '');
				
				if (Hijax.busy)
				{
					Hijax.log.info('Cancel click event because BUSY');
					window.fireEvent('navstillbusy');
				}
				else
				{
					// Only update state if the target URL is different from current URL
					if (this.hasURLChanged(href))
					{
						/**
						 * Implement this in HTML5 and Old mode to provide 
						 */
						this.handleAnchorClick(url);
					}
				}
			}
		}.bind(this));
	},
	
	isExternal: function(e)
	{
        var anchor = e.target.get('tag') === 'a' ? e.target : e.target.getParent('a');
        var isExternal = 'external' === anchor.get('rel');
        console.log('external link?', isExternal);
		return isExternal;
	},
	
	/**
	 * Get the anchor's href and then convert it to the ajax equivalent.
	 * This is the time to turn browser URLs into AJAX URLs.
	 */
	getURL: function(e)
	{
		var anchor = e.target.get('tag') === 'a' ? e.target
				: e.target.getParent('a');

		return anchor.get('href');
	},
	
	/**
	 * Map the URL to the ajax equivalent.
	 * This is the time to turn browser URLs into AJAX URLs.
	 */
	mapURL: function(url)
	{
		// Correct for home page
		url = url === '/' ? '/'+this.options.home : url;
		
		// Translate URL
		var hijaxURL = this.translateURL(url);
		
		Hijax.log.debug('URL ['+url+'] mapped to ['+hijaxURL.href+'] '+
				'level('+hijaxURL.level+') level down('+hijaxURL.levelDown+')');
		
		return hijaxURL;
	},
	
	onStateChanged: function(url)
	{
		if (Hijax.busy)
		{
			Hijax.log.info('onStateChange: PENDING: '+url);
			this.setPendingStateChange(url);
		} 
		else
		{
			Hijax.log.info('onStateChange: HANDLE: '+url);
			// Map the URL.
			var mappedURL = this.mapURL(url);
			// Set the referrer as the current page ready for the next state change.
			this.updateReferrer();
			// Fire a historychange event
			this.fireHistoryChange(url);
			// Handle the state change with the mapped URL.
			this.handleStateChange(mappedURL);
		}
	},
	
	fireHistoryChange: function(url)
	{
		// Fire a historychange event
		window.fireEvent('historychange', url);
	},
	
	/**
	 * Do the ajax work to retrieve the content from the given URL.
	 * No URL translation happens here. All URLs should be AJAX requestable.
	 */
	handleStateChange: function(mappedURL)
	{
		// Reset the request state
		if (this.request)
		{
			this.request.cancel();
		}
		this.request = new Request.HTML();
		
		this.ifContainer(mappedURL.level, function(container){
			Hijax.load(this.request, container, mappedURL, function(){
				Hijax.instance().hijack(container);
			});
		}.bind(this));
	},
	
	/**
	 * Tests if the container with specified level is present on the page. 
	 * If found, callback is called with the container as the argument.
	 * If not found an exception is thrown.
	 */
	ifContainer: function(level, callback)
	{
		var id = this.options.idPrefix + level;
		var container = $(id);
		if (container)
		{
			callback(container);
		}
		else
		{
			throw 'Target container "'+id+'" is not defined.';
		}
	},
	
	/**
	 * Prepares the Current URL and Target URL by normalising and converting to 
	 * arrays, ready for comparison in the form:
	 * 
	 * {
	 *     'current': ['home'],
	 *     'target': ['rates', 'diving']
	 * }
	 */
	getURLChange: function(targetURL)
	{
		var current = this.referrer.replace(/^#/, '')
			.replace(/^\//, '').replace(/\/$/, '').split('/');
	
		var target = targetURL
			.replace(/^\//, '').replace(/\/$/, '').split('/');
				
		return {
			'current': current,
			'target': target
		};
	},
	
	/**
	 * Determine if the target URL is different from the current URL
	 */
	hasURLChanged: function(url)
	{
		var change = this.getURLChange(url);
		Hijax.log.debug(change);
		var current = change.current.join(' ');
		var target = change.target.join(' ');
		return current !== target;
	},
	
	/**
	 * This is the actual translator: edit this to do the translation.
	 * It takes a URL string and returns an object in the form:
	 * 
	 * {
	 *     'href': '/pages/translated_url',
	 *     'level': 1
	 * }
	 */
	translateURL: function(url)
	{
		var change = this.getURLChange(url);
		var current = change.current;
		var target = change.target;
		Hijax.log.debug('Current['+current+'] Target['+target+']');
		
		// Compute level
		var level = 0;
		
		// Traverse the URL array and stop when the two URLs are no longer similar.
		var traverseURLs = function(level)
		{
			return (current && target && current[level] && target[level]
				&& current[level] === target[level]);
		};
		
		// Iterate and test traverse URLs.
		while(traverseURLs(level)) level++;
		
		// Normalise level
		var levelDown = false;
		if (level === target.length)
		{
			level-=1;
			levelDown = true;
		}
		
		// Compute href
		var href = target[0];
		var terminateTranslation = false;
		var matchedHasDynamicContent = null;
		
		for (var i = 1; i < target.length; i++)
		{
			matchedHasDynamicContent = this.findHasDynamicContent(href);
			if (matchedHasDynamicContent) terminateTranslation = true;
			
			href += (i-1 === level || terminateTranslation ? '/' : '_') + target[i];
		}
		
		Hijax.log.debug('Found dynamic content: '+matchedHasDynamicContent);
		Hijax.log.debug('If you expected dynamic content ensure the base URL is present in the Hijax.Abstract.options.hasDynamicContent:');
		Hijax.log.debug(this.options.hasDynamicContent);
		
		// Wrap the results as an object.
		return {
			'href': href,
			'level': level,
			'levelDown': levelDown,
			'dynamicContentOnly' : 
				this.isRequestDynamicContentOnly(level, matchedHasDynamicContent)
		};
	},
	
	findHasDynamicContent: function(href)
	{
		var matchedHasDynamicContent = null;
		this.options.hasDynamicContent.each(function(item){
			if(item == null)
			{
				item = '';
			}
			
			if (href.replace(/_/g, '/') === item.replace(/_/g, '/'))
			{
				matchedHasDynamicContent = item;
			}
		});
		return matchedHasDynamicContent;
	},
	
	isRequestDynamicContentOnly: function(level, matchedHasDynamicContent)
	{
		if (matchedHasDynamicContent)
		{
			var length = matchedHasDynamicContent.split('/').length;
			return level === length;
		}
	}
	
});