<?php defined('SYSPATH') or die('No direct script access.');

class Kohana_Model_Node extends ORM
{

	protected $_table_name = 'app_nodes';
	
	protected $_table_name_options = 'app_options';
	
	protected $_db_group = 'default';
	
	protected $_concrete;
	
	protected $_belongs_to = array(
		'appuser' => array(
			'foreign_key' => 'user_id',
		),
	);
	
// 	protected $_has_many = array (
// 		'groups' => array(
// 			'model'		=> 'group',	
// 			'through' 	=> 'entry_groups',
// 		),
// 	);

	/**
	 * This is the list of field names in the app_node table that are writable
	 * by a user. Created, modified, id, for example are only writable by code.
	 * @var array
	 * @example array('name', 'status', 'order')
	 */
	public static $WRITABLE = array('name', 'status', 'order');
	
	function rules()
	{
		$model = 'Model_'.ucfirst($this->_concrete);
		return call_user_func(array($model, 'node_rules'));
	}

	function labels()
	{
		$model = 'Model_'.ucfirst($this->_concrete);
		return call_user_func(array($model, 'node_labels'));
	}
	
	private $_requested_options = array();
	private $options;
	
	public function __construct($id = NULL){
		$config = Kohana::$config->load('node');
		$this->_has_one = $config->get('_has_one');
		$this->_has_many = $config->get('_has_many');
		parent::__construct($id);
	}
	
	/**
	 * Load the Node with the associated type. 
	 * In order to load an Node with the associated type, entry must know about
	 * the associate via the _has_one relationship which is specified in the config
	 * file.
	 * @see Kohana_ORM::with()
	 */
	public function concrete($target_path = NULL)
	{
		if (isset($target_path))
		{
			$this->_concrete = $target_path;
			if (!array_key_exists($target_path, $this->_has_one))
			{
				throw new Kohana_Exception(
							'":target_path" was not found in the node config file',
				array(':target_path' => $target_path));
			}
			return $this->_with($target_path);
		}
		else
		{
			return $this->_concrete;
		}
	}
	
	/**
	 * Binds another one-to-one object to this model.  One-to-one objects
	 * can be nested using 'object1:object2' syntax
	 *
	 * @param  string $target_path Target model to bind to
	 * @return void
	 */
	public function _with($target_path)
	{
		if (isset($this->_with_applied[$target_path]))
		{
			// Don't join anything already joined
			return $this;
		}

		// Split object parts
		$aliases = explode(':', $target_path);
		$target = $this;
		foreach ($aliases as $alias)
		{
			// Go down the line of objects to find the given target
			$parent = $target;
			$target = $parent->_related($alias);

			if ( ! $target)
			{
				// Can't find related object
				return $this;
			}
		}

		// Target alias is at the end
		$target_alias = $alias;

		// Pop-off top alias to get the parent path (user:photo:tag becomes user:photo - the parent table prefix)
		array_pop($aliases);
		$parent_path = implode(':', $aliases);

		if (empty($parent_path))
		{
			// Use this table name itself for the parent path
			$parent_path = $this->_object_name;
		}
		else
		{
			if ( ! isset($this->_with_applied[$parent_path]))
			{
				// If the parent path hasn't been joined yet, do it first (otherwise LEFT JOINs fail)
				$this->with($parent_path);
			}
		}

		// Add to with_applied to prevent duplicate joins
		$this->_with_applied[$target_path] = TRUE;

		// Use the keys of the empty object to determine the columns
		foreach (array_keys($target->_object) as $column)
		{
			$name = $target_path.'.'.$column;
			$alias = $target_path.':'.$column;

			// Add the prefix so that load_result can determine the relationship
			$this->select(array($name, $alias));
		}

		if (isset($parent->_belongs_to[$target_alias]))
		{
			// Parent belongs_to target, use target's primary key and parent's foreign key
			$join_col1 = $target_path.'.'.$target->_primary_key;
			$join_col2 = $parent_path.'.'.$parent->_belongs_to[$target_alias]['foreign_key'];
		}
		else
		{
			// Parent has_one target, use parent's primary key as target's foreign key
			$join_col1 = $parent_path.'.'.$parent->_primary_key;
			$join_col2 = $target_path.'.'.$parent->_has_one[$target_alias]['foreign_key'];
		}

		// Join the related object into the result
		$this->join(array($target->_table_name, $target_path), 'INNER')->on($join_col1, '=', $join_col2);

		return $this;
	}
	
	/**
	 * Restrict to status=published
	 * @return Model_Entry
	 */
	public function published()
	{
		return $this->where('status', '=', 'published');
	}
	
	/**
	 * Like find_all but loads only published
	 * @return Model_Entry
	 */
	public function find_all_published()
	{
		return $this->published()->find_all();
	}
	
	/**
	 * Like find but loads only published
	 * @return Model_Entry
	 */
	public function find_published()
	{
		return $this->published()->find();
	}
	
	/**
	 * Join with Options to provide ordering
	 * @param string $column
	 * @param string $direction
	 * return Model_Entry
	 */
	public function order_by_option($column, $direction=NULL)
	{
		$prefix = 'option_';
		$alias = $prefix.$column;
		
		return $this->join(array('app_options', $alias), 'LEFT')
			->on($alias.'.node_id', '=', 'node.id')
			->on($alias.'.attribute', '=', DB::expr("'".$column."'"))
			->order_by(DB::expr($alias.'.value * 1'), $direction);
	}
	
	/**
	 * Identifies the type of the Entry
	 * @param string $table in the plural form as it is found in the db schema
	 * @return Model_Entry
	 */
	public function in($table)
	{
		$prefix = 'in_table_';
		$alias = $prefix.$table;
		
		return $this->join(array($table, $alias), 'RIGHT')
			->on($alias.'.entry_id', '=', 'entry.id');
	}
	
	/**
	 * 
	 * Enter description here ...
	 * @param mixed $options
	 * @return Model_Entry
	 */
	public function with_options($options)
	{
		if (is_string($options))
		{
			$options = array($options);
		}
		
		$this->_requested_options = $options;
		
		return $this;
	}
	
	public function where_option($option, $op, $value)
	{
		$result = DB::select('node_id')
			->from($this->_table_name_options)
			->where('attribute', '=', $option)
			->where('value', '=', $value)
			->execute($this->_db_group);
		
		$node_ids = implode(',', array_keys($result->as_array('node_id')));
		$op = count($result) > 1 ? 'IN' : '=';
		
		return $this->where('id', $op, $node_ids);
	}
	
	public function find_all()
	{
		$database_result = parent::find_all();
		$enhanced_database_result = array();
		
		// If the result set is empty then there is no work to do with options
		if (count($database_result) > 0)
		{
			$options = $this->load_options($database_result->as_array('id')); 
			
			foreach ($database_result as $result)
			{
				$matching_options = array();
				foreach ($options as $option)
				{
					if($option['node_id'] === $result->id)
					{
						$matching_options[$option['attribute']] = $option['value'];
					}
				}
				$result->options = $matching_options;
				
				$enhanced_database_result[] = $result;
			}
		}
		return $enhanced_database_result;
	}
	
	public function find()
	{
		$result = parent::find();
		
		$options = $this->load_options($result->id);
		
		$matching_options = array();
		foreach ($options as $option)
		{
			$matching_options[$option['attribute']] = $option['value'];
		}
		$result->options = $matching_options;
		
		return $result;
	}
	
	public function domain_delete($ids = NULL)
	{
		if (is_null($ids))
		{
			return parent::domain_delete();
		}
		elseif (is_array($ids) && count($ids))
		{
			// Delete the objects
			return DB::delete($this->_table_name)
				->where($this->_primary_key, 'IN', $ids)
				->where('domain_id', '=', $this->domain_id)
				->execute($this->_db);
		}
		else
		{
			Log::instance()->add(Log::DEBUG, 'Could not delete '.print_r($ids, 1));
		}
	}
	
	public function option($attribute, $default=NULL)
	{
		if (isset($this->options[$attribute]))
		{
			return $this->options[$attribute];
		}
		return $default;
	}
	
	/**
	 * Useful for debug to see what options are have been loaded
	 */
	public function options()
	{
		return $this->options;
	}
	
	public function values(array $values = NULL, array $expected = NULL)
	{
		// Get
		if (is_null($values))
		{
			$values = array();
			
			$concrete_type = $this->concrete();
			$node_core = $this->as_array();
			$node_concrete = $node_core[$concrete_type];
			$node_options = $this->options();
			unset($node_core[$concrete_type]);
			unset($node_concrete['node_id']);
			
			$this->_values($values, $node_core, '');
			$this->_values($values, $node_concrete, 'node_'.$concrete_type.'_');
			$this->_values($values, $node_options, 'node_options_');
			
			return $values;
		}
		// Set
		else
		{
			$new = ! $this->loaded();
			
			// Populate the core node attributes, concretes and options are done elsewhere
			foreach ($values as $key => $val)
			{
				$pattern = '#^node_core_#';
				if (preg_match($pattern, $key))
				{
					$key = preg_replace($pattern, '', $key);
					$this->{$key} = $val;
				}
			}
			
			// Save changed/create times and user id
			if ($new)
			{
				$this->created = time();
				$this->user_id = Auth::instance()->get_user()->id;
			}
			$this->changed = time();
			
			// If new the concrete type needs to be instantiated
			if ( ! isset($this->{$this->_concrete}))
			{
				$this->{$this->_concrete} = ORM::factory($this->_concrete);
			}
			
			// Set the concrete values
			$this->{$this->_concrete}->values($this->_extract_values($this->_concrete));
			
			// Obtain the option values
			$this->options = $this->_extract_values('options');
			
			return parent::values($values, $expected);
		}
	}
	
	private function _values(&$values, $array, $prefix)
	{
		foreach ($array as $key => $val)
		{
			$values[$prefix.$key] = $val;
		}
	}
	
	/**
	 * Updates or Creates the record depending on loaded()
	 *
	 * @chainable
	 * @param  Validation $validation Validation object
	 * @return ORM
	 */
	public function save(Validation $validation = NULL)
	{
		return Transaction::factory($this->_db_group)
			->call(array($this, 'save'))
			->with($validation)
			->execute();
	}
	
	public function transactional_save(Validation $validation = NULL)
	{
		$new = ! $this->loaded();
	
		// New node must be given required values
		if ($new)
		{
			$user = Auth::instance()->get_user();
			$this->set('user_id', $user ? $user->id : NULL);
			$this->set('domain_id', $this->domain_id);
		}
	
		$exception = new ORM_Validation_Exception('', Validation::factory(array()));
		
		// Save the node and collect validation exceptions
		try
		{
			parent::save($validation);
		} 
		catch (ORM_Validation_Exception $e)
		{
			$exception->merge($e);
		}
	
		// If new attach the concrete type to the node by foreign key
		if ($new)
		{
			$this->{$this->_concrete}->node_id = $this->id;
		}
		
		// Save the concrete type and collect validation exceptions
		try
		{
			$this->{$this->_concrete}->save();
		}
		catch (ORM_Validation_Exception $e)
		{
			$exception->merge($e);
		}
		catch (Database_Exception $e)
		{
			/*
			 * A) If the node didn't save due to a validation exception then the 
			 * concrete's node_id foreign key will be blank which causes a database
			 * exception. 
			 * 
			 * B) If the node saves successfully then $exception is NULL so if we
			 * get a database exception then it's for real so rethrow it.
			 */
			if ( ! is_null($exception))
			{
				throw $exception;
			}
			else
			{
				throw $e;
			}
		}
		
		// Save the options and collect validation exceptions
		try
		{
			// Save the options
			$this->store_options($this->id, $this->options);
		}
		catch (ORM_Validation_Exception $e)
		{
			$exception->merge($e);
		}
		
		// If there were validation exceptions then throw it now
		$errors = $exception->errors();
		if ( ! empty($errors) )
		{
			throw $exception;
		}
	
		return $this;
	}
	
	protected function validate_options()
	{
		$labels = $this->{$this->_concrete}->option_labels();
		$rules = $this->{$this->_concrete}->option_rules();
		
		$options = $this->_extract_values('options');
	
		$validation = Validation::factory($options)
			->labels($labels);
	
		foreach ($rules as $field => $rule)
		{
			$validation->rules($field, $rule);
		}
	
		if ( ! $validation->check())
		{
			throw new ORM_Validation_Exception('options', $validation);
		}
	}
	
	protected function load_options($id)
	{
		if (count($this->_requested_options))
		{
			return DB::select('node_id', 'attribute', 'value')
				->from($this->_table_name_options)
				->where('node_id', is_array($id) ? 'IN' : '=', $id)
				->where('attribute', 'IN', $this->_requested_options)
				->execute($this->_db_group);
		}
		else
		{
			return array();
		}
	}
	
	protected function store_options($id, array $options)
	{
		// Do the validation
		$this->validate_options();
		
		// Find current attributes
		$result = DB::select('attribute', 'value')
			->from($this->_table_name_options)
			->where('node_id', '=', $id)
			->execute($this->_db_group);
		
		DB::delete($this->_table_name_options)
			->where('node_id', '=', $id)
			->execute($this->_db_group);
		
		$current = array();
		foreach($result as $row)
		{
			$current[$row['attribute']] = $row['value'];
		}
		
		foreach ($options as $key => $val)
		{
			$current[$key] = $val;
			
			if (empty($val))
			{
				unset($current[$key]);
			}
		}
		
		$inserts = array();
		foreach ($current as $key => $val)
		{
			$inserts[] = array($id, $key, $val);
		}
		
		if (!empty($inserts))
		{
			// Insert everything with one query.
			$insert = DB::insert($this->_table_name_options, array('node_id', 'attribute', 'value'));
			foreach ($inserts as $row)
			{
				$insert->values($row);
			}
			$insert->execute($this->_db_group);
		}
		
		return $this;
	}
	
	protected function _extract_values($namespace)
	{
		$values = array();
		
		$keys = array_keys($_POST);
		
		foreach ($keys as $key)
		{
			if (preg_match('#^node_'.$namespace.'_(.*)#', $key, $matches))
			{
				$field = $matches[1];
				$values[$field] = $_POST[$matches[0]];
			}
		}
		return $values;
	}
	
// 	/**
// 	 * Initializes the Database Builder to given query type
// 	 *
// 	 * @param  integer $type Type of Database query
// 	 * @return ORM
// 	 */
// 	protected function _build($type)
// 	{
// 		$return = parent::_build($type);
		
// 		if ( ! $this->domain_id_applied )
// 		{
// 			$message = 'Unprotected query: Domain ID not set.';
// 			Log::instance()->add(Log::EMERGENCY, $message);
// 			throw new Kohana_Exception($message);
// 		}
		
// 		return $return;
// 	}

//	public function defaults() {
//		return $this->values(Entry::$entry_default_values);
//	}
	
//  	function defaults() {
// 		return $this->values(Entry::entry_default_values());
// 	}
	
// 	public function require_permissions($permissions){
		
// 		if (empty($permissions))
// 			return $this;
			
// 		$this->and_where_open();
		
// 		foreach ($permissions as $permission){
// 			$this->or_where('permission', 'like', "%$permission%");
// 		}
		
// 		$this->and_where_close();
		
// 		return $this;
// 	}
	
}